@@ -46,19 +46,28 @@ from itertools import takewhile
 import operator
 import os
 import re
-from typing import Optional, Callable, Tuple, Union, List, Iterable, Dict
-
-from debian.changelog import Version
+from typing import (
+    ContextManager,
+    Optional,
+    Callable,
+    Tuple,
+    Union,
+    List,
+    Iterable,
+    Dict,
+)
 
 import subprocess
 import warnings
 
+from debian.changelog import Version
+
 from ._deb822 import PkgRelation
 from .deb822 import (
     Deb822Editor,
     ChangeConflict,
     Deb822Paragraph,
-    parse_deb822_paragraph,
+    parse_deb822_file,
     )
 from .reformatting import GeneratedFile
 
@@ -154,24 +163,28 @@ def pg_buildext_updatecontrol(path: str = '.') -> None:
     subprocess.check_call(["pg_buildext", "updatecontrol"], cwd=path)
 
 
-def guess_template_type(template_path: str) -> Optional[str]:
+def guess_template_type(
+        template_path: str,
+        debian_path: Optional[str] = None) -> Optional[str]:
     """Guess the type for a control template.
 
     Args:
       template_path: Template path
+      debian_path: Path to debian directory
     Returns:
       Name of template type; None if unknown
     """
     # TODO(jelmer): This should use a proper make file parser of some sort..
-    try:
-        with open('debian/rules', 'rb') as f:
-            for line in f:
-                if line.startswith(b'debian/control:'):
-                    return 'rules'
-                if line.startswith(b'debian/%: debian/%.in'):
-                    return 'rules'
-    except FileNotFoundError:
-        pass
+    if debian_path is not None:
+        try:
+            with open(os.path.join(debian_path, 'rules'), 'rb') as f:
+                for line in f:
+                    if line.startswith(b'debian/control:'):
+                        return 'rules'
+                    if line.startswith(b'debian/%: debian/%.in'):
+                        return 'rules'
+        except FileNotFoundError:
+            pass
     try:
         with open(template_path, 'rb') as f:
             template = f.read()
@@ -181,10 +194,16 @@ def guess_template_type(template_path: str) -> Optional[str]:
                 return 'cdbs'
             elif b'PGVERSION' in template:
                 return 'postgresql'
-            elif b'@lintian-brush-test@' in template:
-                return 'lintian-brush-test'
             else:
-                deb822 = parse_deb822_paragraph(template)
+                try:
+                    deb822 = next(iter(
+                        parse_deb822_file(
+                            template.splitlines(),
+                            accept_files_with_error_tokens=True)))
+                except StopIteration:
+                    pass
+                else:
+                    build_depends = ''
                 build_depends = deb822.get('Build-Depends', '')
                 if any(iter_relations(build_depends, 'gnome-pkg-tools')):
                     return 'gnome'
@@ -192,7 +211,8 @@ def guess_template_type(template_path: str) -> Optional[str]:
                     return 'cdbs'
     except IsADirectoryError:
         return 'directory'
-    if os.path.exists('debian/debcargo.toml'):
+    if (debian_path is not None
+            and os.path.exists(os.path.join(debian_path, 'debcargo.toml'))):
         return 'debcargo'
     return None
 
@@ -229,6 +249,14 @@ def _expand_control_template(
         template_path: str, path: str, template_type: str):
     package_root = os.path.dirname(os.path.dirname(path)) or '.'
     if template_type == 'rules':
+        template_mtime = os.stat(template_path).st_mtime
+        try:
+            path_mtime = os.stat(path).st_mtime
+        except FileNotFoundError:
+            path_mtime = None
+        if template_mtime == path_mtime:
+            # Delete path to force it to be regenerated
+            os.unlink(path)
         subprocess.check_call(
             ['./debian/rules', 'debian/control'],
             cwd=package_root)
@@ -236,10 +264,6 @@ def _expand_control_template(
         dh_gnome_clean(package_root)
     elif template_type == 'postgresql':
         pg_buildext_updatecontrol(package_root)
-    elif template_type == 'lintian-brush-test':
-        with open(template_path, 'rb') as inf, open(path, 'wb') as outf:
-            outf.write(
-                inf.read().replace(b'@lintian-brush-test@', b'testvalue'))
     elif template_type == 'directory':
         raise GeneratedFile(path, template_path)
     else:
@@ -248,13 +272,15 @@ def _expand_control_template(
 
 def _update_control_template(
         template_path: str, path: str, changes, expand_template=True):
-    template_type = guess_template_type(template_path)
+    template_type = guess_template_type(
+            template_path, debian_path=os.path.dirname(path))
     if template_type is None:
         raise GeneratedFile(path, template_path)
     if template_type == 'directory':
         # We can't handle these yet
         raise GeneratedFile(path, template_path)
-    with Deb822Editor(template_path) as updater:
+    with Deb822Editor(
+            template_path, accept_files_with_error_tokens=True) as updater:
         resolve_conflict: Optional[Callable[[
             str, str, Optional[str], Optional[str], Optional[str]],
             Optional[str]]]
@@ -325,6 +351,7 @@ class ControlEditor(object):
     """
 
     changed: bool
+    _field_order_preserver: ContextManager
 
     def __init__(self, path: str = 'debian/control',
                  allow_reformatting: Optional[bool] = None,
@@ -413,7 +440,8 @@ class ControlEditor(object):
             template_path = _find_template_path(self.path)
             if template_path:
                 self._template_only = True
-                template_type = guess_template_type(template_path)
+                template_type = guess_template_type(
+                    template_path, debian_path=os.path.dirname(self.path))
                 if template_type is None:
                     raise GeneratedFile(self.path, template_path)
                 _expand_control_template(
@@ -1040,7 +1068,7 @@ class PkgRelationFieldEditor(object):
         Args:
           package: package name
         """
-        if self.value is None:
+        if self._parsed is None:
             return
 
         def keep(relation):
